from github_metrics.helpers import filter_hotfixes


def count_hotfixes(pr_list, exclude_authors, filter_authors):
    if not exclude_authors:
        exclude_authors = []
    hotfix_list = filter_hotfixes(pr_list, exclude_authors, filter_authors)

    print(
        f"""
            \033[1mHotfixes Count\033[0m
            ----------------------------------
            Total PRs counted: {len(hotfix_list)}
            """
    )



from github_metrics.common import extract_datetime_or_none, get_author_login
from github_metrics.helpers import filter_valid_prs


def format_pr_list(pr_list):
    return [
        {
            "author": get_author_login(pr),
            "merged_at": extract_datetime_or_none(pr.get("mergedAt"))
            if pr.get("mergedAt")
            else None,
        }
        for pr in pr_list
    ]


def get_merged_prs(formatted_pr_list):
    merged_prs = []

    for pr in formatted_pr_list:
        if pr["merged_at"] is not None:
            merged_prs.append(pr)
    return merged_prs


def get_prs_authors(pr_list):
    pr_author_list = []

    for pr in pr_list:
        if pr["author"] not in pr_author_list:
            pr_author_list.append(pr["author"])
    return pr_author_list


def call_merge_rate_statistics(
    pr_list, include_hotfixes, exclude_authors, filter_authors
):
    valid_pr_list = filter_valid_prs(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    formatted_pr_list = format_pr_list(valid_pr_list)
    merged_prs = get_merged_prs(formatted_pr_list)
    if not merged_prs or merged_prs == []:
        return "There are no valid PRs to pull this data from, please select another timeframe"

    prs_authors = get_prs_authors(merged_prs)
    merge_rate = len(merged_prs) / len(prs_authors)

    print(
        f"""
            \033[1mMerge Rate\033[0m
            ----------------------------------
            Total PRs calculated: {len(merged_prs)}
            Total devs calculated: {len(prs_authors)}
            ----------------------------------
            Merge Rate: {merge_rate}
            """
    )



import arrow
import numpy

from github_metrics.common import extract_datetime_or_none, get_author_login
from github_metrics.helpers import (
    filter_valid_prs,
    format_timedelta,
    get_time_without_weekend,
)


def get_reviews_from_pr(pr):
    reviews_root = pr.get("reviews")

    if not reviews_root:
        return []

    reviews = reviews_root.get("nodes", [])
    if not reviews:
        return []

    return reviews


def get_first_review(pr):
    pr_author_login = get_author_login(pr)
    reviews = get_reviews_from_pr(pr)

    different_author_reviews = [
        r for r in reviews if pr_author_login != get_author_login(r)
    ]

    if not different_author_reviews:
        return

    return different_author_reviews[0]


def hours_without_review(pr):
    open_date = extract_datetime_or_none(pr["created_at"])

    if pr["first_review_at"] is None:
        time_without_review = arrow.now() - open_date
        return time_without_review.total_seconds() / 3600

    time_without_review = extract_datetime_or_none(pr["first_review_at"]) - open_date
    return time_without_review.total_seconds() / 3600


def filter_prs_with_more_than_24h_before_review(pr_list, use_time_before_review=False):
    return [pr for pr in pr_list if hours_without_review(pr) > 24]


def format_pr_list(pr_list):
    pr_list_with_hours = [
        {
            "title": pr["title"],
            "author": get_author_login(pr),
            "created_at": extract_datetime_or_none(pr.get("createdAt")),
            "first_review_at": extract_datetime_or_none(
                get_first_review(pr).get("createdAt")
            )
            if get_first_review(pr)
            else None,
        }
        for pr in pr_list
    ]

    return pr_list_with_hours


def filter_reviewed_prs(pr_list):
    return [pr for pr in pr_list if pr["first_review_at"] is not None]


def calulate_prs_review_time_statistics(
    pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
):
    valid_pr_list = filter_valid_prs(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    formatted_pr_list = format_pr_list(valid_pr_list)
    reviewed_prs = filter_reviewed_prs(formatted_pr_list)
    prs_more_than_24h_without_review = filter_prs_with_more_than_24h_before_review(
        formatted_pr_list
    )

    review_time_list = []
    for pr in reviewed_prs:
        review_time = pr["first_review_at"] - pr["created_at"]
        if exclude_weekends:
            review_time = get_time_without_weekend(
                pr["created_at"], pr["first_review_at"]
            )
        review_time_list.append(review_time)

    total_prs = len(formatted_pr_list)
    unreviewed_prs = total_prs - len(reviewed_prs)
    prs_over_24h = len(prs_more_than_24h_without_review)

    mean = numpy.mean(review_time_list)
    median = numpy.median(review_time_list)
    percentile = numpy.percentile(review_time_list, 95)

    print(
        f"""
            \033[1mTime to review\033[0m
            ----------------------------------
            Total valid PRs: {len(formatted_pr_list)}
            Unreviewed PRs: {unreviewed_prs} ({round((unreviewed_prs * 100) / total_prs, 2)}%)
            PRs with more than 24h waiting for review: {prs_over_24h} ({round(prs_over_24h * 100 / total_prs, 2)}%)
            ----------------------------------
            Mean: {format_timedelta(mean)} ({round(mean.total_seconds()/3600, 2)} hours)
            Median: {format_timedelta(median)} ({round(median.total_seconds()/3600, 2)} hours)
            95 percentile: {format_timedelta(percentile)} ({round(percentile.total_seconds()/3600, 2)} hours)
        """
    )



import arrow


def extract_datetime_or_none(isoformat_str_date):
    try:
        arrow_date = arrow.get(isoformat_str_date)
    except Exception:
        return None

    return arrow_date.datetime


def get_author_login(pr_or_review):
    author = pr_or_review.get("author")
    if not author:
        return None

    return author.get("login")


def get_reviews_from_pr(pr):
    reviews_root = pr.get("reviews")

    if not reviews_root:
        return []

    reviews = reviews_root.get("nodes", [])
    if not reviews:
        return []

    return reviews



import numpy

from github_metrics.helpers import filter_valid_prs


def format_pr_list(pr_list):
    return [
        {
            "additions": pr["additions"],
            "deletions": pr["deletions"],
        }
        for pr in pr_list
    ]


def call_pr_size_statistics(pr_list, include_hotfixes, exclude_authors, filter_authors):
    valid_pr_list = filter_valid_prs(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    formatted_pr_list = format_pr_list(valid_pr_list)

    total_line_diff = []
    diff_line_rate = []

    for pr in formatted_pr_list:
        total_line_diff.append(pr["additions"] + pr["deletions"])
        diff_line_rate.append(pr["additions"] - pr["deletions"])

    total_mean = numpy.mean(total_line_diff)
    total_median = numpy.median(total_line_diff)
    total_percentile = numpy.percentile(total_line_diff, 95)

    rate_mean = numpy.mean(diff_line_rate)
    rate_median = numpy.median(diff_line_rate)
    rate_percentile = numpy.percentile(diff_line_rate, 95)

    print(
        f"""
            \033[1mPull Requests Size\033[0m
            ----------------------------------
            Total PRs calculated: {len(formatted_pr_list)}
            ----------------------------------
            Total Lines Mean: {round(total_mean, 2)} lines
            Total Lines Median: {round(total_median, 2)} lines
            Total Lines 95 percentile: {round(total_percentile, 2)} lines

            Diff Rate Mean: {round(rate_mean, 2)}
            Diff Rate Median: {round(rate_median, 2)}
            Diff Rate 95 percentile: {round(rate_percentile, 2)}
            """
    )



from github_metrics.common import get_author_login
import datetime


def is_closed(pr):
    return pr.get("closedAt") is not None and not pr.get("mergedAt")


def is_release(pr):
    release_in_head_branch_name = "release" in pr["headRefName"].lower()
    release_in_title = "release" in pr["title"].lower()
    head_branch_is_master = (
        "master" in pr["headRefName"].lower() or "main" in pr["headRefName"].lower()
    )
    base_branch_is_production = "production" in pr["baseRefName"].lower()

    if not base_branch_is_production:
        return False

    return release_in_head_branch_name or head_branch_is_master or release_in_title


def is_hotfix(pr):
    hotfix_in_head_branch_name = (
        "hotfix/" in pr["headRefName"].lower() or "hf/" in pr["headRefName"].lower()
    )
    hotfix_in_title = "hotfix" in pr["title"].lower()
    base_branch_is_production = "production" in pr["baseRefName"].lower()

    if not base_branch_is_production:
        return False

    return hotfix_in_head_branch_name or hotfix_in_title


def is_merge_back_from_prod(pr):
    base_branch_is_master = (
        "master" in pr["baseRefName"].lower() or "main" in pr["baseRefName"].lower()
    )
    head_branch_is_production = "production" in pr["headRefName"].lower()
    return base_branch_is_master and head_branch_is_production


def exclude_closeds(pr_list):
    return [pr for pr in pr_list if not is_closed(pr)]


def exclude_releases(pr_list):
    return [pr for pr in pr_list if not is_release(pr)]


def exclude_hotfixes(pr_list):
    return [pr for pr in pr_list if not is_hotfix(pr)]


def exclude_merge_backs_from_prod(pr_list):
    return [pr for pr in pr_list if not is_merge_back_from_prod(pr)]


def exclude_authors_in_list(pr_list, authors):
    return [pr for pr in pr_list if not get_author_login(pr) in authors]


def filter_authors_in_list(pr_list, authors):
    return [pr for pr in pr_list if get_author_login(pr) in authors]


def filter_valid_prs(pr_list, include_hotfixes, exclude_authors, filter_authors):
    valid_pr_list = exclude_closeds(pr_list)
    valid_pr_list = exclude_releases(valid_pr_list)
    valid_pr_list = exclude_merge_backs_from_prod(valid_pr_list)

    if not include_hotfixes:
        valid_pr_list = exclude_hotfixes(valid_pr_list)
    if exclude_authors:
        valid_pr_list = exclude_authors_in_list(valid_pr_list, exclude_authors)
    if filter_authors:
        valid_pr_list = filter_authors_in_list(valid_pr_list, filter_authors)
    return valid_pr_list


def filter_hotfixes(pr_list, exclude_authors, filter_authors):
    valid_pr_list = exclude_closeds(pr_list)
    valid_pr_list = exclude_releases(valid_pr_list)
    valid_pr_list = exclude_merge_backs_from_prod(valid_pr_list)
    valid_pr_list = [pr for pr in pr_list if is_hotfix(pr)]

    if exclude_authors:
        valid_pr_list = exclude_authors_in_list(valid_pr_list, exclude_authors)
    if filter_authors:
        valid_pr_list = filter_authors_in_list(valid_pr_list, filter_authors)

    return valid_pr_list


def format_timedelta(timedelta):
    if timedelta.total_seconds() < 0:
        return "Invalid timeframe"

    days = timedelta.days
    hours, remainder = divmod(timedelta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    return f"{days} days {hours} hours {minutes} minutes"


def get_weekend_time(start_at, end_at):
    day_count = end_at.date() - start_at.date()
    weekends = datetime.timedelta()
    for i in range(0, day_count.days + 1):
        day = start_at + datetime.timedelta(days=i)

        # 5 represents Saturday and 6 represents Sunday
        if day.weekday() == 5 or day.weekday() == 6:
            # In a time period, if it's starting a time count, count from the beginning until end of day
            if i == 0:
                weekends += (
                    datetime.datetime(day.year, day.month, day.day, 23, 59, 59) - day
                )

            # If it's the end of a time period, count from start of day until the end time
            elif i == day_count.days:
                weekends += end_at - datetime.datetime(day.year, day.month, day.day)

            else:
                weekends += datetime.timedelta(days=1)
    return weekends


def get_time_without_weekend(start_at, end_at):
    weekend_timedelta = get_weekend_time(start_at, end_at)
    timedelta = end_at - start_at
    return timedelta - weekend_timedelta



import numpy

from github_metrics.common import extract_datetime_or_none, get_author_login
from github_metrics.helpers import (
    filter_valid_prs,
    format_timedelta,
    get_time_without_weekend,
)


def get_formatted_list_of_commits(commit_data):
    commits_list = []

    if not commit_data.get("edges"):
        return []

    for data in commit_data.get("edges"):
        commit = data.get("node").get("commit")
        commits_list.append(
            {
                "message": commit.get("message"),
                "commited_at": extract_datetime_or_none(commit.get("committedDate")),
            }
        )
    return commits_list


def format_pr_list(pr_list):
    return [
        {
            "title": pr["title"],
            "author": get_author_login(pr),
            "created_at": extract_datetime_or_none(pr.get("createdAt")),
            "merged_at": extract_datetime_or_none(pr.get("mergedAt"))
            if pr.get("mergedAt")
            else None,
            "closed_at": extract_datetime_or_none(pr.get("closedAt"))
            if pr.get("closedAt")
            else None,
            "commits": get_formatted_list_of_commits(pr.get("commits")),
        }
        for pr in pr_list
    ]


def call_time_to_open_statistics(
    pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
):
    valid_pr_list = filter_valid_prs(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    formatted_pr_list = format_pr_list(valid_pr_list)

    if not formatted_pr_list or formatted_pr_list == []:
        return "There are no valid PRs to pull this data from, please select another timeframe"

    time_to_open = []
    for pr in formatted_pr_list:
        first_commit_time = pr["commits"][0]["commited_at"]
        timedelta = pr["created_at"] - first_commit_time
        if exclude_weekends:
            timedelta = get_time_without_weekend(first_commit_time, pr["created_at"])
        time_to_open.append(timedelta)

    mean = numpy.mean(time_to_open)
    median = numpy.median(time_to_open)
    percentile = numpy.percentile(time_to_open, 95)

    print(
        f"""
            \033[1mTime to open\033[0m
            ----------------------------------
            Total PRs calculated: {len(formatted_pr_list)}
            ----------------------------------
            Mean: {format_timedelta(mean)} ({round(mean.total_seconds()/3600, 2)} hours)
            Median: {format_timedelta(median)} ({round(median.total_seconds()/3600, 2)} hours)
            95 percentile: {format_timedelta(percentile)} ({round(percentile.total_seconds()/3600, 2)} hours)
            """
    )



import numpy

from github_metrics.common import extract_datetime_or_none, get_author_login
from github_metrics.helpers import (
    filter_valid_prs,
    format_timedelta,
    get_time_without_weekend,
)


def get_merged_prs(formatted_pr_list):
    merged_prs = []

    for pr in formatted_pr_list:
        if pr["merged_at"] is not None:
            merged_prs.append(pr)
    return merged_prs


def format_pr_list(pr_list):
    pr_list_with_hours = [
        {
            "title": pr["title"],
            "author": get_author_login(pr),
            "created_at": extract_datetime_or_none(pr.get("createdAt")),
            "merged_at": extract_datetime_or_none(pr.get("mergedAt"))
            if pr.get("mergedAt")
            else None,
        }
        for pr in pr_list
    ]

    return pr_list_with_hours


def calulate_prs_open_to_merge_time_statistics(
    pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
):
    valid_pr_list = filter_valid_prs(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    formatted_pr_list = format_pr_list(valid_pr_list)
    merged_pr_list = get_merged_prs(formatted_pr_list)

    review_time_list = []

    for pr in merged_pr_list:
        open_pr_duration = pr["merged_at"] - pr["created_at"]
        if exclude_weekends:
            open_pr_duration = get_time_without_weekend(
                pr["created_at"], pr["merged_at"]
            )
        review_time_list.append(open_pr_duration)

    mean = numpy.mean(review_time_list)
    median = numpy.median(review_time_list)
    percentile = numpy.percentile(review_time_list, 95)

    print(
        f"""
            \033[1mOpen to Merge\033[0m
            ----------------------------------
            Merged PRs count: {len(merged_pr_list)}
            Valid Merged PRs rate: {round((len(merged_pr_list) * 100) / len(valid_pr_list), 2)}%
            ----------------------------------
            Mean: {format_timedelta(mean)} ({round(mean.total_seconds()/3600, 2)} hours)
            Median: {format_timedelta(median)} ({round(median.total_seconds()/3600, 2)} hours)
            95 percentile: {format_timedelta(percentile)} ({round(percentile.total_seconds()/3600, 2)} hours)
        """
    )



import os

from decouple import config

BASE_DIR = os.path.dirname(__file__)

GITHUB_LOGIN = config("GITHUB_LOGIN", default="")
GITHUB_TOKEN = config("GITHUB_TOKEN", default="")
ORG_NAME = config("ORG_NAME", default="")
REPOSITORY_NAME = config("REPOSITORY_NAME", default="")



import click
import arrow
import os
from github_metrics.metrics.time_to_merge import call_mean_time_to_merge_statistics
from github_metrics.metrics.time_to_review import calulate_prs_review_time_statistics
from github_metrics.metrics.time_to_open import call_time_to_open_statistics
from github_metrics.metrics.merge_rate import call_merge_rate_statistics
from github_metrics.metrics.pr_size import call_pr_size_statistics
from github_metrics.metrics.hotfixes_count import count_hotfixes
from github_metrics.metrics.open_to_merge import (
    calulate_prs_open_to_merge_time_statistics,
)
from github_metrics.metrics.all import call_all_metrics


from github_metrics.request import fetch_prs_between
from github_metrics.settings import BASE_DIR


def setup_enviroment_variables():
    github_username = input("Github username: ")
    github_token = input("Github token: ")
    organization_name = input("Name of the organization: ")
    repository_name = input("Repository name: ")

    with open(os.path.join(BASE_DIR, ".env"), "w") as file:
        content = """GITHUB_LOGIN="{github_username}"\nGITHUB_TOKEN="{github_token}"\nORG_NAME="{organization_name}"\nREPOSITORY_NAME="{repository_name}"\n""".format(
            github_username=github_username,
            github_token=github_token,
            organization_name=organization_name,
            repository_name=repository_name,
        )
        file.write(content)


@click.command()
@click.option(
    "--setup",
    is_flag=True,
    default=False,
    help="Setup your enviroment variables",
)
@click.option(
    "--metric",
    type=str,
    help="""The reference of the metric you'd like to run:
    
    \b
    ttm - Time to Merge
    ttr - Time to Review
    tto - Time to Open
    otm - Open To Merge Time
    mr - Merge Rate
    pr_size - Pull Request Size
    hotfixes_count - Hotfixes Count
    \b""",
)
@click.option(
    "--start-date",
    type=str,
    help="""The metric start date.
    
    Date in format YYYY-mm-dd""",
)
@click.option(
    "--end-date",
    type=str,
    help="""The metric cutoff date.
    
    Date in format YYYY-mm-dd""",
)
@click.option(
    "--include-hotfixes",
    is_flag=True,
    default=False,
    help="Will include all hotfixes in the metric calculation.",
)
@click.option(
    "--exclude-authors",
    type=str,
    help="""
        List of PR authors separated by a comma to be removed from metric.

        eg.: username,other_username""",
)
@click.option(
    "--filter-authors",
    type=str,
    help="""
        Will calculate prs created only by the authors listed in here.

        eg.: username,other_username""",
)
@click.option(
    "--exclude-weekends",
    is_flag=True,
    default=False,
    help="Will exclude weekends from time metric.",
)
def cli(
    metric,
    start_date,
    end_date,
    include_hotfixes,
    exclude_authors,
    filter_authors,
    exclude_weekends,
    setup,
):
    """
    Generates metrics from Github API.
    """

    if setup:
        setup_enviroment_variables()
        return

    start_date = arrow.get(start_date)
    end_date = arrow.get(f"{end_date}T23:59:59")

    exclude_user_list = []
    if exclude_authors:
        exclude_user_list = exclude_authors.split(",")

    filter_user_list = []
    if filter_authors:
        filter_user_list = filter_authors.split(",")

    pr_list = fetch_prs_between(start_date, end_date)
    if metric == "ttm":
        call_mean_time_to_merge_statistics(
            pr_list,
            include_hotfixes,
            exclude_user_list,
            filter_user_list,
            exclude_weekends,
        )
    elif metric == "ttr":
        calulate_prs_review_time_statistics(
            pr_list,
            include_hotfixes,
            exclude_user_list,
            filter_user_list,
            exclude_weekends,
        )
    elif metric == "tto":
        call_time_to_open_statistics(
            pr_list,
            include_hotfixes,
            exclude_user_list,
            filter_user_list,
            exclude_weekends,
        )
    elif metric == "otm":
        calulate_prs_open_to_merge_time_statistics(
            pr_list,
            include_hotfixes,
            exclude_user_list,
            filter_user_list,
            exclude_weekends,
        )
    elif metric == "mr":
        call_merge_rate_statistics(
            pr_list, include_hotfixes, exclude_user_list, filter_user_list
        )
    elif metric == "pr_size":
        call_pr_size_statistics(
            pr_list, include_hotfixes, exclude_user_list, filter_user_list
        )
    elif metric == "hotfixes_count":
        count_hotfixes(pr_list, exclude_user_list, filter_user_list)
    else:
        call_all_metrics(
            pr_list,
            include_hotfixes,
            exclude_user_list,
            filter_user_list,
            exclude_weekends,
        )



import requests
from requests.auth import HTTPBasicAuth
from time import sleep

from github_metrics.common import extract_datetime_or_none
from github_metrics.settings import (
    GITHUB_LOGIN,
    GITHUB_TOKEN,
    REPOSITORY_NAME,
    ORG_NAME,
)


class ClientError(BaseException):
    pass


def format_request_for_github(cursor=None):
    after = ""
    if cursor:
        after = f', after: "{cursor}"'

    return """{{
    organization(login: "{ORG_NAME}") {{
        repository(name: "{REPOSITORY_NAME}") {{
            pullRequests(
                first: 100,
                orderBy: {{
                    field: CREATED_AT,
                    direction: DESC
                }}{after}
            ) {{
                pageInfo {{
                    endCursor
                    startCursor
                    hasNextPage
                }}
                nodes {{
                    id
                    title
                    createdAt
                    baseRefName
                    headRefName
                    additions
                    deletions
                    reviews(first: 10) {{
                        nodes {{
                            createdAt
                            state
                            author {{
                                login
                            }}
                        }}
                    }}
                    author {{
                        login
                    }}
                    mergedAt
                    closedAt
                    commits(first: 100) {{
                        edges {{
                            node {{
                                commit {{
                                    oid
                                    message
                                    committedDate
                                }}
                            }}
                        }}
                    }}
                }}
            }}
        }}
    }}
}}""".format(
        after=after, ORG_NAME=ORG_NAME, REPOSITORY_NAME=REPOSITORY_NAME
    )


def pr_was_created_between(pr, start_date, end_date):
    open_date = extract_datetime_or_none(pr.get("createdAt"))
    return open_date >= start_date and open_date <= end_date


def fetch_prs_between(start_date, end_date):
    if not all([GITHUB_LOGIN, GITHUB_TOKEN, ORG_NAME, REPOSITORY_NAME]):
        raise EnvironmentError(
            "The environment is not properly configured. "
            "Please check if you .env file is created and has the proper variables."
        )
    pr_list = []
    current_date = None
    cursor = None
    has_next_page = True
    while has_next_page and (not current_date or current_date > start_date):
        response = requests.post(
            "https://api.github.com/graphql",
            auth=HTTPBasicAuth(GITHUB_LOGIN, GITHUB_TOKEN),
            json={"query": format_request_for_github(cursor)},
        )

        if response.json().get("errors", False):
            raise ClientError(response.json().get("errors"))

        data = response.json().get("data")

        if not data:
            has_next_page = False
            continue

        organization = data.get("organization")
        if not organization:
            has_next_page = False
            continue

        repository = organization.get("repository")
        if not repository:
            has_next_page = False
            continue

        prs = repository.get("pullRequests")
        if not prs:
            has_next_page = False
            continue

        page_info = prs.get("pageInfo")
        if not page_info:
            has_next_page = False
            continue

        has_next_page = page_info["hasNextPage"]
        cursor = page_info["endCursor"]

        page_pr_list = prs.get("nodes")
        if not page_pr_list:
            has_next_page = False
            continue

        current_date = extract_datetime_or_none(page_pr_list[-1]["createdAt"])
        pr_list += [
            pr
            for pr in page_pr_list
            if pr_was_created_between(pr, start_date, end_date)
        ]
        sleep(1)
    return pr_list



from setuptools import setup, find_packages
from pkg_resources import parse_requirements

with open("requirements.txt") as requirements_file:
    requirements = [
        str(req) for req in parse_requirements(requirements_file.readlines())
    ]

with open("README.md", "r", encoding="utf-8") as readme_file:
    readme = readme_file.read()


setup(
    name="github_metrics",
    version="0.0.10",
    description="Generate development metrics using github data for your project.",
    url="https://github.com/vintasoftware/github-metrics",
    author="Victoria Pantoja (Vinta Software)",
    author_email="victoria.pantoja@vinta.com.br",
    python_requires=">=3.8",
    py_modules=["github_metrics"],
    install_requires=requirements,
    include_package_data=True,
    packages=find_packages(exclude=("tests*",)),
    entry_points="""
        [console_scripts]
        github_metrics=github_metrics.run:cli
    """,
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
)



from github_metrics.metrics.time_to_merge import call_mean_time_to_merge_statistics
from github_metrics.metrics.time_to_review import calulate_prs_review_time_statistics
from github_metrics.metrics.time_to_open import call_time_to_open_statistics
from github_metrics.metrics.open_to_merge import (
    calulate_prs_open_to_merge_time_statistics,
)
from github_metrics.metrics.merge_rate import call_merge_rate_statistics
from github_metrics.metrics.pr_size import call_pr_size_statistics
from github_metrics.metrics.hotfixes_count import count_hotfixes


def call_all_metrics(
    pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
):
    call_mean_time_to_merge_statistics(
        pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
    )
    calulate_prs_review_time_statistics(
        pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
    )
    call_time_to_open_statistics(
        pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
    )
    calulate_prs_open_to_merge_time_statistics(
        pr_list, include_hotfixes, exclude_authors, filter_authors, exclude_weekends
    )
    call_merge_rate_statistics(
        pr_list, include_hotfixes, exclude_authors, filter_authors
    )
    call_pr_size_statistics(pr_list, include_hotfixes, exclude_authors, filter_authors)
    count_hotfixes(pr_list, exclude_authors, filter_authors)
